@@ -3,11 +3,10 @@ import sys
 import time
 
 from fabric.api import env, local, sudo
-from fabric.colors import blue, cyan, green, magenta, red, white, yellow
+from fabric.colors import blue, cyan, green, magenta, red, white, yellow  # noqa
 from fabric.context_managers import settings as fabric_settings
-from fabric.contrib.files import contains, exists
-from fabric.contrib.project import rsync_project
-from fabric.decorators import task, parallel
+from fabric.contrib.files import exists
+from fabric.decorators import task
 from fabric.utils import abort
 
 from jinja2 import Template
@@ -17,6 +16,16 @@ from bootmachine import known_hosts
 import settings
 
 
+@task
+def configure(match="'*'"):
+    """
+    Run salt state.highstate on hosts that match.
+    Usage:
+        fab master configurator.configure
+    """
+    highstate(match)
+
+
 @task
 def highstate(match="'*'"):
     """
@@ -41,34 +50,27 @@ def upload_saltstates():
     """
     if env.host != env.master_server.public_ip:
         abort("tried to upload salttates on a non-master server")
+    if not exists(settings.REMOTE_STATES_DIR, use_sudo=True):
+        sudo("mkdir --parents {0}".format(settings.REMOTE_STATES_DIR))
 
     # catch rsync issue with emacs autosave files
-    for path in (settings.SALTSTATES_DIR, settings.PILLAR_DIR):
+    temp_files = []
+    for path in (settings.LOCAL_STATES_DIR, settings.LOCAL_PILLARS_DIR):
         for match in ("#*#", ".#*"):
-            if local('find {0} -name "{1}"'.format(path, match), capture=True):
-                abort("A temp file matching '{0}' exists in the {1} directory.".format(match, path))
+            temp_file = local('find {0} -name "{1}"'.format(path, match), capture=True)
+            if temp_file:
+                temp_files.append(temp_file)
+    if temp_files:
+        print(red("Temp files must not exist in the saltstates or pillars dirs."))
+        for temp_file in temp_files:
+            print(yellow("found: {0}".format(temp_file)))
+        abort("Found temp files in the saltstates or pillars dirs.")
 
     # rsync pillar and salt files to the fabric users local directory
-
-    rsync_project(local_dir=settings.SALTSTATES_DIR, remote_dir="./salt/", delete=True,
-                  extra_opts="--compress --copy-links", ssh_opts="-o StrictHostKeyChecking=no")
-    rsync_project(local_dir=settings.PILLAR_DIR, remote_dir="./pillar/", delete=True,
-                  extra_opts="--compress --copy-links", ssh_opts="-o StrictHostKeyChecking=no")
-
-    # backup the jinja2 created pillar files
-    if exists("/srv/pillar/salt.sls"):
-        sudo("mv /srv/pillar/salt.sls /srv/salt.sls")
-    if exists("/srv/pillar/users.sls"):
-        sudo("mv /srv/pillar/users.sls /srv/users.sls")
-    # delete the pillar and state files
-    sudo("rm -rf /srv/salt && rm -rf /srv/pillar")
-    # copy the rsynced versions over as root
-    sudo("cp -r ./salt /srv/salt && cp -r ./pillar /srv/pillar")
-    # put the jinja2 created pillar files back in place
-    if exists("/srv/salt.sls"):
-        sudo("mv /srv/salt.sls /srv/pillar/salt.sls")
-    if exists("/srv/users.sls"):
-        sudo("mv /srv/users.sls /srv/pillar/users.sls")
+    local('rsync -a -e "ssh -p {0}" --rsync-path="sudo rsync" {1} {2}@{3}:{4}'.format(
+        env.port, settings.LOCAL_STATES_DIR, env.user, env.host, settings.REMOTE_STATES_DIR))
+    local('rsync -a -e "ssh -p {0}" --rsync-path="sudo rsync" {1} {2}@{3}:{4}'.format(
+        env.port, settings.LOCAL_PILLARS_DIR, env.user, env.host, settings.REMOTE_PILLARS_DIR))
 
 
 @task
@@ -81,24 +83,50 @@ def pillar_update():
     if env.host != env.master_server.public_ip:
         abort("tried to pillar_update on a non-master server")
 
-    pillar_dir = settings.PILLAR_DIR
-    bootmachine_sls_j2 = Template(open("{0}bootmachine.sls.j2".format(pillar_dir), "r", 0).read())
-    bootmachine_sls = open("{0}bootmachine.sls".format(pillar_dir), "w", 0)
+    local_pillars_dir = settings.LOCAL_PILLARS_DIR
+    remote_pillars_dir = settings.REMOTE_PILLARS_DIR
+    if not exists(remote_pillars_dir, use_sudo=True):
+        sudo("mkdir --parents {0}".format(remote_pillars_dir))
+    bootmachine_sls_j2 = Template(
+        open(os.path.join(local_pillars_dir, "bootmachine.sls.j2"), "r", 0).read())
+    bootmachine_sls = open(os.path.join(local_pillars_dir, "bootmachine.sls"), "w", 0)
     bootmachine_sls.write(bootmachine_sls_j2.render(
         bootmachine_servers=env.bootmachine_servers,
+        salt_remote_states_dir=settings.REMOTE_STATES_DIR,
+        salt_remote_pillars_dir=remote_pillars_dir,
         saltmaster_hostname=settings.MASTER,
         saltmaster_public_ip=env.master_server.public_ip,
         saltmaster_private_ip=env.master_server.private_ip,
-        ssh_port = settings.SSH_PORT,
-        ssh_users = settings.SSH_USERS,
+        ssh_port=settings.SSH_PORT,
+        ssh_users=settings.SSH_USERS,
+        salt_aur_pkgver=settings.SALT_AUR_PKGVER,
+        salt_aur_pkgrel=settings.SALT_AUR_PKGREL,
     ))
+
     # TODO: only upload and refresh when file has changes
+    home_dir = local("eval echo ~${0}".format(env.user), capture=True)
+    if exists(home_dir, use_sudo=True):
+        scp_dir = home_dir
+    else:
+        scp_dir = "/tmp/"
     try:
-        local("scp -P {0} {1}bootmachine.sls {2}@{3}:/tmp/bootmachine.sls".format(env.port, pillar_dir, env.user, env.host))
+        local("scp -P {0} {1} {2}@{3}:{4}".format(
+              env.port,
+              os.path.join(local_pillars_dir, "bootmachine.sls"),
+              env.user,
+              env.host,
+              os.path.join(scp_dir, "bootmachine.sls")))
     except:
         known_hosts.update(env.host)
-        local("scp -P {0} {1}bootmachine.sls {2}@{3}:/tmp/bootmachine.sls".format(env.port, pillar_dir, env.user, env.host))
-    sudo("mv /tmp/bootmachine.sls /srv/pillar/bootmachine.sls")
+        local("scp -P {0} {1} {2}@{3}:$(eval echo ~${4})bootmachine.sls".format(
+              env.port,
+              os.path.join(local_pillars_dir, "bootmachine.sls"),
+              env.user,
+              env.host,
+              scp_dir))
+    sudo("mv {0} {1}".format(
+        os.path.join(scp_dir, "bootmachine.sls"),
+        os.path.join(remote_pillars_dir, "bootmachine.sls")))
     sudo("salt '*' saltutil.refresh_pillar &")  # background because it hangs on debian 6
 
 
@@ -128,15 +156,17 @@ def update_master_iptables():
         match = sudo("iptables -nvL | grep {0}".format(port))
         for server in env.bootmachine_servers:
             if server.private_ip not in match:
-                sudo("iptables -I INPUT {0} -s {1} -m state --state new -m tcp -p tcp --dport {2} -j ACCEPT".format(
-                    insert_line[0], server.private_ip, port))
+                sudo("iptables -I INPUT {0} -s {1} -m state --state new -m tcp -p tcp \
+                --dport {2} -j ACCEPT".format(insert_line[0], server.private_ip, port))
 
 
 @task
 def launch():
     """
     After the salt packages are installed, accept the new minions,
-    upload states, and call highstate.
+    upload states.
+    Usage:
+        fab master configurator.launch
     """
     if env.host != env.master_server.public_ip:
         abort("tried to launch on a non-master server")
@@ -149,8 +179,7 @@ def launch():
 
     time.sleep(10)  # sleep a little to give minions a chance to become visible
     accept_minions()
-
-    highstate()
+    local("fab master configurator.restartall")
 
 
 @task
@@ -158,23 +187,33 @@ def accept_minions():
     """
     Accept salt-key's for all minions.
     Usage:
-        fab master salt.accept_minions
+        fab master configurator.accept_minions
     """
     if env.host != env.master_server.public_ip:
         abort("tried to accept minions on a non-master server")
 
-    accepted = filter(None, sudo("salt-key --raw-out --list acc").translate(None, "'[]\ ").split(","))
-
+    def __get_accepted_minions():
+        """TODO: remove when all distros support salt 0.10.5"""
+        try:
+            accepted = eval(sudo("salt-key --out raw --list acc"))
+        except:
+            accepted = eval(sudo("salt-key --raw-out --list acc"))
+        if type(accepted) == dict:
+            return accepted["minions"]
+        else:
+            return accepted  # support salt version < 0.10.5
+    minions = __get_accepted_minions()
     slept = 0
-    while len(accepted) != len(settings.SERVERS):
-        unaccepted = [s["servername"] for s in settings.SERVERS if s["servername"] not in accepted]
+
+    while len(minions) != len(settings.SERVERS):
+        unaccepted = [s["servername"] for s in settings.SERVERS if s["servername"] not in minions]
 
         with fabric_settings(warn_only=True):
             for server in unaccepted:
                 sudo("salt-key --quiet --accept={0}".format(server))
-        accepted = filter(None, sudo("salt-key --raw-out --list acc").translate(None, "'[]\ ").split(","))
-        if len(accepted) != len(settings.SERVERS):
-            local("fab each configurator.restart")
+        minions = __get_accepted_minions()
+        if len(minions) != len(settings.SERVERS):
+            local("fab master configurator.restartall")
             time.sleep(5)
             slept += 5
             print(yellow("there are still unaccpeted keys, trying again."))
@@ -189,10 +228,10 @@ def list_minions():
     """
     List all minions.
     Usage:
-        fab master salt.list_minions
+        fab master configurator.list_minions
     """
     if env.host != env.master_server.public_ip:
-        abort("tried to list minions on a non-master server")
+        abort("tried to list minions from a non-master server")
 
     sudo("salt-key --list all")
 
@@ -204,28 +243,29 @@ def change_master_ip(ip_address):
 
 
 @task
-@parallel
-def restart():
+def restartall():
     """
-    Restart all salt masters and minion daemons.
-    Simply wrap salt's reestart method cor the chose distro.
+    Restart first the salt master than all minion daemons.
+    Usage:
+        fab master configurator.restartall
     """
-    server = [s for s in env.bootmachine_servers if s.public_ip == env.host][0]
-    env.servername = server.name
-    env.port = server.port
-    env.user = server.user
+    if env.host != env.master_server.public_ip:
+        abort("tried to restartall from a non-master server")
 
-    for server in settings.SERVERS:
-        if env.servername == server["servername"]:
-            distro_module = server["distro_module"]
-
-    try:
-        __import__(distro_module)
-        distro = sys.modules[distro_module]
-    except ImportError:
-        abort("Unable to import the module: {0}".format(distro_module))
-
-    distro.restart_salt()
+    for server in env.bootmachine_servers:
+        stop(__getdistro_setenv(server.name))
+    time.sleep(5)
+    start(__getdistro_setenv(env.master_server.name))
+    for server in env.bootmachine_servers:
+        if server.name != env.master_server.name:
+            start(__getdistro_setenv(server.name))
+    env.servername = env.master_server.name
+    env.host = server.public_ip
+    env.host_string = "{0}:{1}".format(env.master_server.public_ip,
+                                       env.master_server.port)
+    env.hosts = [env.master_server.public_ip]
+    env.port = env.master_server.port
+    env.user = env.master_server.user
 
 
 def revoke(servername):
@@ -258,3 +298,31 @@ def start(distro):
     Simply wrap salt's start method for the chosen distro.
     """
     distro.start_salt()
+
+
+def stop(distro):
+    """
+    Stop the salt master and minion daemons.
+    Simply wrap salt's stop method for the chosen distro.
+    """
+    distro.stop_salt()
+
+
+def __getdistro_setenv(servername):
+    """
+    """
+    server = [s for s in env.bootmachine_servers
+              if s.name == servername][0]
+    env.servername = server.name
+    env.host = server.public_ip
+    env.host_string = "{0}:{1}".format(server.public_ip, server.port)
+    env.hosts = [server.public_ip]
+    env.port = server.port
+    env.user = server.user
+    distro_module = [s for s in settings.SERVERS
+                     if s["servername"] == server.name][0]["distro_module"]
+    try:
+        __import__(distro_module)
+        return sys.modules[distro_module]
+    except ImportError:
+        abort("Unable to import the module: {0}".format(distro_module))
