@@ -7,7 +7,7 @@ import novaclient.v1_1
 from fabric.api import env, local
 from fabric.contrib import console
 from fabric.decorators import task
-from fabric.colors import blue, cyan, green, magenta, red, white, yellow
+from fabric.colors import blue, cyan, green, magenta, red, white, yellow  # noqa
 from fabric.operations import prompt
 from fabric.utils import abort
 
@@ -136,8 +136,10 @@ def boot(servername, image, flavor=2, servers=None):
             abort("Image not found. Run ``openstack image-list`` to view options.")
 
     print(green("... sending boot request to rackspace for ``{name}``".format(name=servername)))
-    local("nova boot --image={image_id} --flavor={flavor} {name} --file /root/.ssh/authorized_keys={key}".format(
-           image_id=image_id, flavor=flavor, name=servername, key=settings.SSH_PUBLIC_KEY))
+    local("nova boot --image={image_id} --flavor={flavor} --file /root/.ssh/authorized_keys={key} \
+    --nic net-id=00000000-0000-0000-0000-000000000000 \
+    --nic net-id=11111111-1111-1111-1111-111111111111 {name}".format(
+        image_id=image_id, flavor=flavor, name=servername, key=settings.SSH_PUBLIC_KEY))
     env.new_server_booted = True
 
 
@@ -156,11 +158,14 @@ def bootem(servers=None):
     for server in servers:
         boot(server["servername"], server["image"], server["flavor"], env.bootmachine_servers)
 
+    long_initial_sleep = 75
+    print(yellow("... waiting {0}s for new servers to be assigned IP adddress.".format(
+        long_initial_sleep)))
+    time.sleep(long_initial_sleep)
     print(yellow("... verifying that all servers are ``ACTIVE``."))
-
     env.bootmachine_servers = list_servers(as_list=True)
 
-    slept = 0
+    slept = long_initial_sleep
     sleep_interval = 10
     status = None
     while status is None:
@@ -200,8 +205,8 @@ def destroy(servername, destroy_all=False, force=False):
 
     # calling list_servers immediately after deleting returns the deleted servers
     if not destroy_all:
-        time.sleep(5)
         env.bootmachine_servers = list_servers(as_list=True)
+        print("It may take up to a minute for a server to be completely deleted.")
 
 
 @task
@@ -213,13 +218,14 @@ def destroyem(force=False):
     """
     if not force:
         servers = ", ".join([s["servername"] for s in settings.SERVERS])
-        reply = prompt("Permanently destroying *EVERY* server:\n{0}\n\nAre you sure? y/N".format(servers))
+        reply = prompt("Permanently destroying *EVERY* server:\n{0}\n\nAre you sure? y/N".format(
+            servers))
         if reply is not "y":
             abort("Did not destroy *ANY* servers")
 
     for server in settings.SERVERS:
         destroy(server["servername"], destroy_all=True)
-    time.sleep(5)
+    print("It may take up to a minute for a server to be completely deleted.")
 
 
 def set_bootmachine_servers(roles=None, ip_type="public", append_port=True):
@@ -231,12 +237,13 @@ def set_bootmachine_servers(roles=None, ip_type="public", append_port=True):
 
     for server in env.bootmachine_servers:
         if server.status != "ACTIVE":
-            console.confirm("The server `{0}` is not `ACTIVE` and is in the `{1}` phase. Continue?".format(server.name, server.status))
+            console.confirm("The server `{0}` is not `ACTIVE` and is in the `{1}` phase. \
+Continue?".format(server.name, server.status))
 
     for server in env.bootmachine_servers:
         # Verify (by name) that the live server was defined in the settings.
         try:
-            instance = [n for n in settings.SERVERS if n["servername"] == server.name][0]
+            [n for n in settings.SERVERS if n["servername"] == server.name][0]
         except IndexError:
             continue
         # If a ``roles`` list was passed in, verify it identically matches the server's roles.
@@ -253,5 +260,6 @@ def set_bootmachine_servers(roles=None, ip_type="public", append_port=True):
                 ips.append(server.public_ip + ":" + str(settings.SSH_PORT))
         else:
             ips.append(server.addresses[ip_type][0])
-        server.distro_module = [n["distro_module"] for n in settings.SERVERS if n["servername"] == server.name][0]
+        server.distro_module = [n["distro_module"] for n in settings.SERVERS
+                                if n["servername"] == server.name][0]
     return ips
