@@ -1,21 +1,22 @@
-from fabric.api import env, local
+from fabric.api import local
 
 
-def add(public_ip):
+def add(user, public_ip, port):
     """
     Permanently add server to the local list of known hosts, but
     first remove any matching keys to prevent conflicts.
     """
     if search(public_ip):
         remove(public_ip)
-    local("ssh {user}@{ip_addr} -o StrictHostKeyChecking=no &".format(user=env.user, ip_addr=public_ip))
+    local('ssh -q {user}@{ip_addr} -p {port} -o StrictHostKeyChecking=no "echo 2>&1"'.format(
+        user=user, ip_addr=public_ip, port=port))
 
 
-def update(public_ip):
+def update(user, public_ip, port):
     """
     Updating a key is the same as adding.
     """
-    add(public_ip)
+    add(public_ip, port)
 
 
 def search(public_ip):
@@ -24,10 +25,7 @@ def search(public_ip):
     returning any occurrences found.
     """
     result = local("ssh-keygen -H -F {0}".format(public_ip), capture=True)
-    if result:
-        return True
-    else:
-        return False
+    return bool(result)
 
 
 def remove(public_ip):
